from typing import List, Dict

from whiskerrag_types.interface.retriever_interface import BaseRetriever
from whiskerrag_types.model.retrieval import RetrievalChunk
from whiskerrag_utils.registry import RegisterTypeEnum, register


@register(RegisterTypeEnum.RETRIEVER, "simple")
class SimpleRetriever(BaseRetriever):
    chunk_list: List[RetrievalChunk]
    chunk_index: Dict[str, RetrievalChunk]

    def __init__(self, chunk_list: List[RetrievalChunk]):
        self.chunk_list = chunk_list
        self.chunk_index = self._build_index()

    def _build_index(self) -> Dict[str, str]:
        return {chunk.chunk_id: chunk for chunk in self.chunk_list}

    def retrieve(self, query: str, top_k: int = 5) -> List[Dict[str, str]]:
        results = []
        for doc_id, content in self.doc_index.items():
            if query.lower() in content.lower():
                results.append(
                    {
                        "id": doc_id,
                        "content": content,
                        "score": 1.0,
                    }
                )

        return sorted(results, key=lambda x: x["score"], reverse=True)[:top_k]


if __name__ == "__main__":
    pass



from typing import List
from langchain_text_splitters import CharacterTextSplitter, MarkdownTextSplitter
from langchain_core.documents import Document
from langchain_openai import OpenAIEmbeddings

from whiskerrag_types.interface.embed_interface import BaseEmbedding
from whiskerrag_types.model.chunk import Chunk
from whiskerrag_types.model.knowledge import (
    EmbeddingModelEnum,
    Knowledge,
    KnowledgeTypeEnum,
)
from whiskerrag_utils.registry import RegisterTypeEnum, register


@register(RegisterTypeEnum.EMBEDDING, EmbeddingModelEnum.OPENAI)
class OpenAIEmbedding(BaseEmbedding):
    async def embed(
        self, knowledge: Knowledge, documents: List[Document]
    ) -> List[Chunk]:
        print(f"start embed knowledge: {knowledge}")
        splitter = None
        if knowledge.knowledge_type == KnowledgeTypeEnum.TEXT:
            splitter = CharacterTextSplitter(
                chunk_size=knowledge.split_config.get("chunk_size"),
                chunk_overlap=knowledge.split_config.get("chunk_overlap"),
            )
        if knowledge.knowledge_type == KnowledgeTypeEnum.MARKDOWN:
            splitter = MarkdownTextSplitter(
                chunk_size=knowledge.split_config.get("chunk_size"),
                chunk_overlap=knowledge.split_config.get("chunk_overlap"),
            )
        if splitter is None:
            raise Exception("not support knowledge type")

        chunks: List[Chunk] = []
        docs = splitter.split_documents(documents)
        embeddings = OpenAIEmbeddings()
        for doc in docs:
            print(f"doc: {doc.page_content}")
            embedding = embeddings.embed_query(doc.page_content)
            print(f"embedding result: {embedding}")
            chunk = Chunk(
                context=doc.page_content,
                metadata={
                    **doc.metadata,
                },
                embedding=embedding,
                knowledge_id=knowledge.knowledge_id,
                embedding_model_name=knowledge.embedding_model_name,
                space_id=knowledge.space_id,
            )
            chunks.append(chunk)
        return chunks



from abc import ABC, abstractmethod
from typing import Optional, TypeVar, Type, List
from pydantic import BaseModel
from whiskerrag_types.model.chunk import Chunk
from whiskerrag_types.model.knowledge import EmbeddingModelEnum
from whiskerrag_types.model.retrieval import RetrievalChunk

from .settings_interface import SettingsInterface
from .logger_interface import LoggerManagerInterface
from ..model import Task, Knowledge, Tenant, PageParams, PageResponse

T = TypeVar("T", bound=BaseModel)


class DBPluginInterface(ABC):
    settings: SettingsInterface
    logger: LoggerManagerInterface

    def __init__(self, logger: LoggerManagerInterface, settings: SettingsInterface):
        logger.info("DB plugin is initializing...")
        self.settings = settings
        self.logger = logger
        self.init()
        logger.info("DB plugin is initialized")

    @abstractmethod
    async def init(self):
        pass

    @abstractmethod
    async def get_db_client(self):
        pass

    @abstractmethod
    async def save_knowledge_list(
        self, knowledge_list: List[Knowledge]
    ) -> List[Knowledge]:
        pass

    @abstractmethod
    async def get_knowledge_list(
        self, space_id: str, page_params: PageParams
    ) -> PageResponse[Knowledge]:
        pass

    @abstractmethod
    async def get_knowledge(self, knowledge_id: str) -> Knowledge:
        pass

    @abstractmethod
    async def search_space_chunk_list(
        self,
        space_id_list: List[str],
        query_embedding: List[float],
        query_embedding_model_name: EmbeddingModelEnum,
        metadata_filter: Optional[dict],
    ) -> List[RetrievalChunk]:
        pass

    @abstractmethod
    async def search_knowledge_chunk_list(
        self,
        knowledge_id_list: List[str],
        query_embedding: List[float],
        query_embedding_model_name: EmbeddingModelEnum,
        metadata_filter: Optional[dict],
    ) -> List[RetrievalChunk]:
        pass

    @abstractmethod
    async def update_knowledge(self, knowledge: Knowledge):
        pass

    @abstractmethod
    async def delete_knowledge(self, knowledge_id_list: List[str]):
        pass

    @abstractmethod
    async def save_chunk_list(self, chunks: List[Chunk]):
        pass

    @abstractmethod
    async def get_chunk_by_knowledge_id(self, chunk_id: str) -> Chunk:
        pass

    @abstractmethod
    async def save_task_list(self, task_list: List[Task]):
        pass

    @abstractmethod
    async def update_task_list(self, task_list: List[Task]):
        pass

    @abstractmethod
    async def get_tenant_by_id(self, tenant_id: str):
        pass

    @abstractmethod
    async def validate_tenant_by_sk(self, secret_key: str) -> bool:
        pass

    @abstractmethod
    async def get_tenant_by_sk(self, secret_key: str) -> Tenant | None:
        pass

    @abstractmethod
    def get_paginated_data(
        self,
        table_name: str,
        model_class: Type[T],
        page_params: PageParams,
    ) -> PageResponse[T]:
        pass
